import { Component, ViewContainerRef, forwardRef, Input, OnInit, OnChanges } from '@angular/core';
import { NG_VALUE_ACCESSOR, ControlValueAccessor } from '@angular/forms';

export const INPUT_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => InputComponent),
  multi: true
};

@Component({
  selector: 'ui-kit-input',
  templateUrl: './input.component.html',
  styleUrls: ['../../../styles/css/input.css'],
  providers: [INPUT_CONTROL_VALUE_ACCESSOR]
})
export class InputComponent implements OnInit, OnChanges {
  @Input() placeholder: string = '';
  @Input() type: string = '';
  @Input() prefix: string = '';
  @Input() label: string = '';
  @Input() inputType: string = 'text';

  private active: boolean = false;
  private class: string;
  private internalValue: any;

  private onTouchedCallback: () => void = () => { };
  private onChangeCallback: (_: any) => void = () => { };

  ngOnInit() {
    this.initClass();
  }

  ngOnChanges() {
    this.initClass();
  }

  initClass() {
    let prefixClass = this.prefix ? 'prefix' : '';
    let activeClass = this.active ? 'is-active' : '';
    this.class = `ui-kit-form-input-container ${this.type} ${prefixClass} ${activeClass}`;
  }

  get value(): any {
    return this.internalValue;
  };

  set value(value: any) {
    if (value !== this.internalValue) {
      this.internalValue = value;
      this.onChangeCallback(value);
    }
  }

  writeValue(value: any) {
    if (value !== this.internalValue) {
      this.internalValue = value;
    }
  }

  focus() {
    this.active = true;
    this.initClass();
  }

  onBlur() {
    this.active = false;
    this.initClass();
    this.onTouchedCallback();
  }

  registerOnChange(fn: any) {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any) {
    this.onTouchedCallback = fn;
  }
}
